@@ -44,8 +44,12 @@
 
 #define WINDOW_BORDER_WIDTH 1
 
+struct SigGlobals_s
+{
+    pid_t childPid;
+};
 
-static pid_t pid=-1;
+typedef struct SigGlobals_s SigGlobals_t;
 
 struct AppData_s
 {
@@ -58,6 +62,9 @@ struct AppData_s
 
 typedef struct AppData_s AppData_t;
 
+
+
+static SigGlobals_t sig_globals;
 static GC gc_white;
 static GC gc_black;
 static GC gc_onColor;
@@ -108,7 +115,7 @@ static int redraw(Display * dpy, Window win, int mouseClickPos,
      /***** play ******/
 
      drawPlayButton(dpy, win, &base, buttonsize,
-            ((pid != -1) && (!appData->paused)) ? gc_onColor : gc_offColor,
+            ((sig_globals.childPid > 0) && (!appData->paused)) ? gc_onColor : gc_offColor,
                                            gc_black, gc_white, buttonDown == 0);
 #if 0
      drawProgressBar(dpy, win, &base, buttonsize, gc_onColor, gc_black, gc_white, appData->dlProgress);
@@ -117,13 +124,13 @@ static int redraw(Display * dpy, Window win, int mouseClickPos,
      /***** pause *****/
      base.x += buttonsize;
      drawPauseButton(dpy, win, &base, buttonsize,
-             ((pid != -1) && appData->paused) ? gc_onColor : gc_offColor,
+             ((sig_globals.childPid > 0) && appData->paused) ? gc_onColor : gc_offColor,
                                            gc_black, gc_white, buttonDown == 1);
 
      /***** stop *****/
      base.x += buttonsize;
      drawStopButton(dpy, win, &base, buttonsize,
-             (pid == -1) ? gc_onColor : gc_offColor, gc_black, gc_white,
+             (sig_globals.childPid > 0) ? gc_onColor : gc_offColor, gc_black, gc_white,
                                                                buttonDown == 2);
      return buttonDown;
 }
@@ -156,15 +163,15 @@ static int igetenv(char *var, int def)
 static void my_play(AppData_t * appData)
 {
      appData->paused = 0;
-     if(pid != -1)
+     if(sig_globals.childPid > 0)
      {
-          if(!kill(-pid, SIGCONT))
+          if(!kill(-sig_globals.childPid, SIGCONT))
           {
                return;
           }
      }
 
-     pid = spawn_app(appData->command, appData->cmd_flags);
+     sig_globals.childPid = spawn_app(appData->command, appData->cmd_flags);
 }
 
 /**
@@ -174,9 +181,9 @@ static void my_play(AppData_t * appData)
  */
 static void my_pause(AppData_t * appData)
 {
-     if(pid != -1)
+     if(sig_globals.childPid > 0)
      {
-          if(!kill(-pid, SIGSTOP))
+          if(!kill(-sig_globals.childPid, SIGSTOP))
           {
                appData->paused = 1;
           }
@@ -188,9 +195,9 @@ static void my_pause(AppData_t * appData)
  */
 static void low_die(void)
 {
-     if(pid != -1)
+     if(sig_globals.childPid > 0)
      {
-          kill_app(pid);
+          kill_app(sig_globals.childPid);
      }
      _exit(0);
 }
@@ -202,17 +209,16 @@ static void low_die(void)
  */
 static void my_stop(AppData_t * appData)
 {
-     if(pid == -1)
+     if(sig_globals.childPid > 0)
      {
-         return;
+          if(appData->paused)
+          {
+               kill(-sig_globals.childPid, SIGCONT);
+               appData->paused = 0;
+          }
+          kill_app(sig_globals.childPid);
+          sig_globals.childPid = -1;
      }
-     if(appData->paused)
-     {
-          kill(-pid, SIGCONT);
-          appData->paused = 0;
-     }
-     kill_app(pid);
-     pid = -1;
 }
 
 /**
@@ -435,7 +441,7 @@ static void check_pipe_fd_events(Display * dpy, Window win,
      if((n == 0) || ((n < 0) && (errno != EINTR)))
      {
           D("Pipe returned n=%i\n", n);
-          terminate(pid, dpy);
+          terminate(sig_globals.childPid, dpy);
           exit(EX_UNAVAILABLE);
      }
 
@@ -660,10 +666,9 @@ int main(int argc, char **argv)
      XSetWindowAttributes attr;
      Display * dpy = 0;
      Window topLevel;
-     int sig_chld_fd;
-     int rd_chld_fd = -1;
      int pipe_fd;
      int repeatsLeft;
+     int sig_chld_fd;
 
      if(!readCommandLine(argc, argv, &width,
                                  &height,
@@ -675,7 +680,7 @@ int main(int argc, char **argv)
      }
 
 
-    repeatsLeft = appData.repeats;
+     repeatsLeft = appData.repeats;
 
      if(!(dpy = XOpenDisplay(getenv("DISPLAY"))))
      {
@@ -734,8 +739,8 @@ int main(int argc, char **argv)
      XSetIOErrorHandler(die2);
      XSetErrorHandler(die);
 
-     sig_chld_fd = redirect_SIGCHLD_to_fd();
 
+     sig_globals.childPid = -1;
      signal(SIGHUP, sigdie);
      signal(SIGINT, sigdie);
      signal(SIGTERM, sigdie);
@@ -743,13 +748,15 @@ int main(int argc, char **argv)
      if(igetenv("autostart",1))
      {
           my_play(&appData);
-          rd_chld_fd = get_chld_out_fd();
      }
 
+     sig_chld_fd = redirect_SIGCHLD_to_fd();
+
      while(1)
      {
           fd_set fds;
           int maxFd;
+          int rd_chld_fd = get_chld_out_fd();
 
           check_x_events(dpy, topLevel, &appData);
 
@@ -791,13 +798,13 @@ int main(int argc, char **argv)
           }
           D("SELECT OUT\n");
 
-          if(pid != -1)
+          if(sig_globals.childPid > 0)
           {
                int status;
-               if(waitpid(pid, &status, WNOHANG) > 0)
+               if(waitpid(sig_globals.childPid, &status, WNOHANG) > 0)
                {
                     appData.paused = 0;
-                    pid = -1;
+                    sig_globals.childPid = -1;
                     if(appData.repeats != INF_LOOPS)
                     {
                          repeatsLeft--;
