@@ -1524,15 +1524,15 @@ static void check_all_events(Display * dpy, SwallowMutex_t * mutex)
 
           if(selectRetVal > 0)
           {
-	       if (FD_ISSET(pipe_fd, &fds))
+	       if((pipe_fd >= 0) && FD_ISSET(pipe_fd, &fds))
                {
 	            check_pipe_fd_events(dpy, mutex);
                }
-               if( FD_ISSET(sig_chld_fd, &fds))
+               if((sig_chld_fd >= 0) && FD_ISSET(sig_chld_fd, &fds))
                {
                     handle_SIGCHLD_event();
                }
-               if( FD_ISSET(rd_chld_fd, &fds))
+               if((rd_chld_fd >= 0) && FD_ISSET(rd_chld_fd, &fds))
                {
                     handle_chld_out_event(rd_chld_fd);
                }
