@@ -1326,6 +1326,54 @@ static size_t max_string_len(const char * str1, const char * str2)
     return (len1 > len2) ? len1 : len2;
 }
 
+#ifdef READ_FROM_INT_BLOB
+extern unsigned char _binary_mozplugger_so_start[];
+extern unsigned char _binary_mozplugger_so_end[];
+unsigned char * blob_pos;
+#endif
+
+
+void * open_mozplugger_so(const char * path)
+{
+#ifdef READ_FROM_INT_BLOB
+    LOG_INFO("- using internal blob as source\n");
+    LOG_DEBUG("open_mozplugger_so %p\n", _binary_mozplugger_so_start);
+    blob_pos = _binary_mozplugger_so_start;
+    return (void *) &blob_pos;
+#else
+    LOG_INFO("- using '%s' as source\n", path);
+    return (void *) fopen(path, "rb");
+#endif
+}
+
+void close_mozplugger_so(void * handle)
+{
+#ifndef READ_FROM_INT_BLOB
+    fclose((FILE *)handle);
+#endif
+}
+
+
+static size_t read_mozplugger_so(char * buffer, size_t bufLen, void * in)
+{
+#ifdef READ_FROM_INT_BLOB
+    unsigned char ** ppBlob = (unsigned char **)in;
+    size_t len = _binary_mozplugger_so_end - *ppBlob;
+    if(len > 0)
+    {
+/*        NOTICE("read_mozplugger_so %p %u\n", *ppBlob, len); */
+        if(len > bufLen)
+            len = bufLen;
+        memcpy(buffer, *ppBlob, len);
+        *ppBlob += len;
+    }
+    return len;
+#else
+    FILE * inFp = (FILE *)in;
+    return fread(buffer, 1, bufLen, inFp);
+#endif
+}
+
 /**
  * Copy mozplugger.so from in to out and add in the magic value
  *
@@ -1336,27 +1384,27 @@ static size_t max_string_len(const char * str1, const char * str2)
 static int copy(const char * in, const char * out, const char * magic)
 {
     char buffer[BUF_LEN];
-    FILE * inFp;
+    void * inFp;
     FILE * outFp;
     bool found = false;
     size_t extra;
     size_t len;
 
-    LOG_INFO("Copy '%s' to '%s' for '%s'\n", in, out, magic);
+    LOG_INFO("Creating '%s' for '%s'\n", out, magic);
 
-    if( !(inFp = fopen(in, "rb")))
+    if( !(inFp = open_mozplugger_so(in)))
     {
         NOTICE("Failed to open '%s'\n", in);
         return 0;
     }
     if( !(outFp = fopen(out, "wb")))
     {
-        fclose(inFp);
+        close_mozplugger_so(inFp);
         ERROR("Failed to open '%s'\n", out);
     }
 
     extra = max_string_len(PLACE_HOLDER_STR, magic)+1;
-    len = fread(buffer, 1, BUF_LEN, inFp);
+    len = read_mozplugger_so(buffer, BUF_LEN, inFp);
 
     while(1)
     {
@@ -1378,15 +1426,15 @@ static int copy(const char * in, const char * out, const char * magic)
 
         len -= fwrite(buffer, 1, BUF_LEN-extra, outFp);
         memcpy(buffer, &buffer[BUF_LEN-extra], len);
-        len += fread(&buffer[len], 1, BUF_LEN - len, inFp);
+        len += read_mozplugger_so(&buffer[len], BUF_LEN - len, inFp);
     }
 
      fwrite(buffer, 1, len, outFp);
-     while((len = fread(buffer, 1, 1024, inFp)) > 0)
+     while((len = read_mozplugger_so(buffer, 1024, inFp)) > 0)
      {
           fwrite(buffer, 1, len, outFp);
      }
-     fclose(inFp);
+     close_mozplugger_so(inFp);
      fclose(outFp);
      return 1;
 }
@@ -1405,7 +1453,9 @@ static void install(const char * localPluginDir, int cfgIdx, int pluginIdx)
      char tmpPluginPath[MAX_FILE_PATH_LEN];
      char globalPluginPath[MAX_FILE_PATH_LEN];
      char magic[MAX_PLUGIN_MAGIC_LEN];
-     char * p = GLOBAL_PLUGIN_DIRS;
+     char * p = GLOBAL_PLUGIN_DIRS "/usr/lib/mozilla/plugins /usr/lib/netscape/plugins "
+                                   "/usr/lib/firefox/plugins /usr/lib/chromium/plugins "
+                                   "/usr/local/lib/browser_plugins/mozplugger";
 
      snprintf(tmpPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger.tmp", localPluginDir);
      snprintf(localPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger%i.so", localPluginDir, pluginIdx);
