@@ -665,6 +665,22 @@ static void run(data_t * const THIS, const char *file, int pipeFd)
 
 }
 
+static char * NP_strdup2(const char * str, int len)
+{
+     char * dupStr = NPN_MemAlloc(len + 1);
+     if(dupStr != NULL)
+     {
+          strncpy(dupStr, str, len);
+          dupStr[len] = '\0';
+     }
+     else
+     {
+          D("NPN_MemAlloc failed, size=%i\n", len+1);
+     }
+     return dupStr;
+}
+
+
 /**
  * String dup function that uses NPN_MemAlloc as opposed to malloc
  *
@@ -677,17 +693,7 @@ static void run(data_t * const THIS, const char *file, int pipeFd)
  */
 static char * NP_strdup(const char * str)
 {
-     const int len = strlen(str);
-     char * dupStr = NPN_MemAlloc(len + 1);
-     if(dupStr != NULL)
-     {
-          strcpy(dupStr, str);
-     }
-     else
-     {
-          D("NPN_MemAlloc failed, size=%i\n", len+1);
-     }
-     return dupStr;
+     return NP_strdup2(str, strlen(str));
 }
 
 
@@ -981,14 +987,13 @@ static bool safeName(const char* name, int isURL)
  *
  * @return None
  */
-static void parseHeaders(data_t * const THIS, const char * headers, char * fileName,
-                                                                 int maxFileNameLen)
+static char * parseHeaders(data_t * const THIS, const char * headers, char * fileName)
 {
      const char * p = headers;
 
      if (!headers)
      {
-          return;
+          return fileName;
      }
 
      while((p = strstr(p, "Content-Disposition:")) != NULL)
@@ -1011,11 +1016,15 @@ static void parseHeaders(data_t * const THIS, const char * headers, char * fileN
 
 	  if(len > 0)
           {
-		strncpy(fileName, start, len);
-		fileName[len] = '\0';
+                if(fileName)
+                {
+                     NPN_MemFree(fileName);
+                }
+                fileName = NP_strdup2(start, len);
 	  }
 	  p += len;
      }
+     return fileName;
 }
 
 /**
@@ -1028,9 +1037,9 @@ static void parseHeaders(data_t * const THIS, const char * headers, char * fileN
  * @param[out] fileName The file Name extracted from the URL
  * @param[in] maxFileNameLen The max length of fileName
  *
- * @return None
+ * @return Pointer to allocated memory with fileName
  */
-static void parseURL(data_t * const THIS, char * fileName, int maxFileNameLen)
+static char * parseURL(data_t * const THIS, int extract_filename)
 {
      const char * frag = strchr(THIS->url, '#');
 
@@ -1046,7 +1055,7 @@ static void parseURL(data_t * const THIS, char * fileName, int maxFileNameLen)
           THIS->urlFragment = NP_strdup(&frag[1]);
      }
 
-     if(fileName)
+     if(extract_filename)
      {
           const char * end = strrchr(THIS->url, '?');
           const char * start;
@@ -1068,14 +1077,9 @@ static void parseURL(data_t * const THIS, char * fileName, int maxFileNameLen)
                start++;
           }
           len = end-start;
-          if(len > maxFileNameLen)
-          {
-               start = end - maxFileNameLen;
-               len = maxFileNameLen;
-          }
-          strncpy(fileName, start, len);
-          fileName[len] = '\0';
+          return NP_strdup2(start, len);
      }
+     return NULL;
 }
 
 /**
@@ -2392,29 +2396,41 @@ static void escapeBadChars(char * string)
 static int guessTmpFile(const char * fileName, int soFar,
                                      char * tmpFilePath, int maxTmpFilePathLen)
 {
-     int i;
+     int i = 0;
      int fd = -1;
-     const int spaceLeft = maxTmpFilePathLen - soFar;
+     int spaceLeft = maxTmpFilePathLen - soFar - 1;
+     const int maxNameLen = pathconf(tmpFilePath, _PC_NAME_MAX);
+     const int fileNameLen = strlen(fileName);
 
-     for(i = 0; i <= 100; i++)
+     if(spaceLeft > maxNameLen)
      {
-          if(i == 0)
+         spaceLeft = maxNameLen;
+     }
+     tmpFilePath[soFar++] = '/';
+
+     while(1)
+     {
+          if(i < 100)
 	  {
-	       strncpy(&tmpFilePath[soFar], fileName, spaceLeft);
+               int n = 0;
+               int pos = 0;
+               if(i > 0)
+               {
+	            n = snprintf(&tmpFilePath[soFar], spaceLeft, "%03i-", i);
+               } 
+	       if(fileNameLen > (spaceLeft-n))
+               {
+                    pos = fileNameLen - (spaceLeft-n);
+               }
+	       strcpy(&tmpFilePath[soFar+n], &fileName[pos]);
           }
-	  else if (i == 100)
+	  else
 	  {
                strncpy(&tmpFilePath[soFar], "XXXXXX", spaceLeft);
                fd = mkstemp(tmpFilePath);
 	       break;
 	  }
-	  else
-	  {
-	       snprintf(&tmpFilePath[soFar], spaceLeft,
-				 "%03i-%s", i, fileName);
-	  }
 
-          tmpFilePath[maxTmpFilePathLen-1] = '\0';
           escapeBadChars(&tmpFilePath[soFar]);
 
 	  fd = open(tmpFilePath, O_CREAT | O_EXCL | O_WRONLY,
@@ -2423,6 +2439,7 @@ static int guessTmpFile(const char * fileName, int soFar,
 	  {
 	       break;
 	  }
+          i++;
      }
 
      return fd;
@@ -2437,32 +2454,30 @@ static int guessTmpFile(const char * fileName, int soFar,
  *
  * @return -1 on error or file descriptor
  */
-static int createTmpFile(const char * fileName,
-                                     char * tmpFilePath, int maxTmpFilePathLen)
+static int createTmpFile(char ** pFileName)
 {
+     char tmpFilePath[512];
      int fd = -1;
      const char * root;
      const pid_t pid = getpid();
 
-     D("Creating temp file for '%s'\n", fileName);
+     D("Creating temp file for '%s'\n", *pFileName);
 
      root = getenv("MOZPLUGGER_TMP");
      if(root)
      {
           int soFar;
 
-          strncpy(tmpFilePath, root, maxTmpFilePathLen);
+          strncpy(tmpFilePath, root, sizeof(tmpFilePath)-1);
           soFar = strlen(tmpFilePath);
 
-	  soFar += snprintf(&tmpFilePath[soFar], maxTmpFilePathLen-soFar,
+	  soFar += snprintf(&tmpFilePath[soFar], sizeof(tmpFilePath)-soFar,
                                                                 "/tmp-%i", pid);
           if( (mkdir(tmpFilePath, S_IRWXU) == 0) || (errno == EEXIST))
           {
                D("Creating temp file in '%s'\n", tmpFilePath);
 
-               tmpFilePath[soFar++] = '/';
-
-	       fd = guessTmpFile(fileName, soFar, tmpFilePath, maxTmpFilePathLen);
+	       fd = guessTmpFile(*pFileName, soFar, tmpFilePath, sizeof(tmpFilePath)-1);
           }
      }
 
@@ -2474,7 +2489,7 @@ static int createTmpFile(const char * fileName,
                root = "/tmp";
           }
 
-          snprintf(tmpFilePath, maxTmpFilePathLen, "%s/mozplugger-%i",
+          snprintf(tmpFilePath, sizeof(tmpFilePath), "%s/mozplugger-%i",
                                                                      root, pid);
           if((mkdir(tmpFilePath, S_IRWXU) == 0) || (errno == EEXIST))
           {
@@ -2482,15 +2497,19 @@ static int createTmpFile(const char * fileName,
 
                D("Creating temp file in '%s'\n", tmpFilePath);
 
-               tmpFilePath[soFar++] = '/';
-
-	       fd = guessTmpFile(fileName, soFar, tmpFilePath, maxTmpFilePathLen);
+	       fd = guessTmpFile(*pFileName, soFar, tmpFilePath, sizeof(tmpFilePath)-1);
           }
      }
+     NPN_MemFree(*pFileName);
 
      if(fd >= 0)
      {
           D("Opened temporary file '%s'\n", tmpFilePath);
+          *pFileName = NP_strdup(tmpFilePath);
+     }
+     else
+     {
+          *pFileName = NULL;
      }
      return fd;
 }
@@ -2510,7 +2529,7 @@ static int createTmpFile(const char * fileName,
 NPError NPP_NewStream(NPP instance, NPMIMEType type, NPStream *stream,
 		      NPBool seekable, uint16_t *stype)
 {
-     char fileName[512];
+     char * fileName = NULL;
      char * savedMimetype = NULL;
      data_t * THIS;
      char refind_command = 0;
@@ -2588,25 +2607,23 @@ NPError NPP_NewStream(NPP instance, NPMIMEType type, NPStream *stream,
           D("Mime type %s\n", type);
      }
 
-     /* Extract from the URL the various additional information */
-     parseURL(THIS, fileName, sizeof(fileName)/sizeof(char)-1);
-     D("fileName (pre-header parse) = %s\n", fileName);
-
-     /* Extract the fileName from HTTP headers, overide URL fileName */
-     parseHeaders(THIS, stream->headers, fileName, sizeof(fileName)/sizeof(char)-1);
-     D("fileName = %s\n", fileName);
-
      if(THIS->command == 0)
      {
 	  reportError(instance, "MozPlugger: No appropriate application found.");
 	  return NPERR_GENERIC_ERROR;
      }
 
+     /* Extract from the URL the various additional information */
+     fileName = parseURL(THIS, 1);
+     D("fileName (pre-header parse) = %s\n", fileName);
+
+     /* Extract the fileName from HTTP headers, overide URL fileName */
+     fileName = parseHeaders(THIS, stream->headers, fileName);
+     D("fileName = %s\n", fileName);
+
      if( (THIS->command->flags & H_STREAM) == 0)
      {
-          char tmpfile[512];
-
-          THIS->tmpFileFd = createTmpFile(fileName, tmpfile, sizeof(tmpfile));
+          THIS->tmpFileFd = createTmpFile(&fileName);
 
           if(THIS->tmpFileFd < 0)
           {
@@ -2617,12 +2634,13 @@ NPError NPP_NewStream(NPP instance, NPMIMEType type, NPStream *stream,
           {
                /* Make file read only by us only */
                fchmod(THIS->tmpFileFd, 0400);
-               THIS->tmpFileName = NP_strdup(tmpfile);
+               THIS->tmpFileName = fileName;
                THIS->tmpFileSize = 0;
           }
      }
      else
      {
+          NPN_MemFree(fileName);
           new_child(instance, THIS->url, 1);
      }
 
@@ -2786,7 +2804,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
           }
 
           /* Extract from the URL the various additional information */
-          parseURL(THIS, 0, 0);
+          (void) parseURL(THIS, 0);
 
 	  new_child(instance, THIS->url, 1);
           THIS->url = NULL; /* Stops new_child from being called again */
